searchData={"items":[{"type":"module","title":"Breeze.Server","doc":"This module powers the GenServer responsible for running the application.\n\nConsider the following Breeze Application:\n\n```\ndefmodule Demo do\n  use Breeze.View\n\n  def mount(_opts, term), do: {:ok, assign(term, counter: 0)}\n\n  def render(assigns) do\n    ~H\" Counter: <%= @counter %> \"\n  end\n\n  def handle_event(_, %{\"key\" => \"ArrowUp\"}, term) do\n    {:noreply, assign(term, counter: term.assigns.counter + 1)}\n  end\n\n  def handle_event(_, %{\"key\" => \"ArrowDown\"}, term) do\n    {:noreply, assign(term, counter: term.assigns.counter - 1)}\n  end\n\n  def handle_event(_, %{\"key\" => \"q\"}, term) do\n    {:stop, term}\n  end\n\n  def handle_event(_, _, term) do\n    {:noreply, term}\n  end\nend\n```\n\nThis can be started directly with:\n\n```\nBreeze.Server.start_link(view: Focus)\n```\n\nOr in a supervision tree:\n\n```\nchildren = [\n  {Breeze.Server, view: Demo}\n]\n```","ref":"Breeze.Server.html"},{"type":"function","title":"Breeze.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Breeze.Server.html#child_spec/1"},{"type":"function","title":"Breeze.Server.start_link/1","doc":"Start the Breeze application.\n\nValid options are:\n\n  * `:view` - the view to run. This is required\n  * `:hide_cursor` - hide the cursor on start. Defaults to `false`","ref":"Breeze.Server.html#start_link/1"},{"type":"module","title":"Breeze.View","doc":"A Breeze View is a process that handles events, updates its states and renders\nto the terminal. Breeze Views are inspired by Phoenix LiveView, but not 100%\ncompatible.\n\n> #### Warning {: .warning}\n>\n> This API is unstable and very likely to change.","ref":"Breeze.View.html"},{"type":"module","title":"Usage - Breeze.View","doc":"The module can be used by including `use Breeze.View`:\n\n```\ndefmodule Demo do\n  use Breeze.View\nend\n```","ref":"Breeze.View.html#module-usage"},{"type":"module","title":"Initial state - Breeze.View","doc":"The initial state can be set in the mount callback:\n\n```\ndef mount(_opts, term), do: {:ok, assign(term, counter: 0)}\n```","ref":"Breeze.View.html#module-initial-state"},{"type":"module","title":"Rendering - Breeze.View","doc":"Rendering is performed using Heex.\n\n\n```\ndef render(assigns) do\n  ~H\" Counter: <%= @counter %> \"\nend\n```\n\nThere are a handful of supported attributes:\n\n\n* `id` - the id of the element. This is required for focusables and implicits\n* `focusable` - if the element should be added to the focus tree. These are added in\nthe order they appear, and can be toggled using tab/shift-tab. The `focus` style\n   alias BackBreeze.Style\nstate can be used to style these. E.g. style=\"border focus:border-3\"\n* `style` - the style for the box. This is covered in the [Style](`m:Breeze.View#module-style`) section.\n* `implicit` - this is a module that will be used for implicit state. This is covered\n in the [Implicits](`m:Breeze.View#module-implicits`) section.","ref":"Breeze.View.html#module-rendering"},{"type":"module","title":"Handling events - Breeze.View","doc":"Any events that come from the terminal or an implicit are handled in the `handle_event/3` callback:\n\n```\ndef handle_event(_, %{\"key\" => \"ArrowUp\"}, term) do\n  {:noreply, assign(term, counter: term.assigns.counter + 1)}\nend\n\ndef handle_event(_, %{\"key\" => \"ArrowDown\"}, term) do\n  {:noreply, assign(term, counter: term.assigns.counter - 1)}\nend\n\ndef handle_event(_, %{\"key\" => \"q\"}, term) do\n  {:stop, term}\nend\n\ndef handle_event(_, _, term) do\n  {:noreply, term}\nend\n```\n\nFor convenience, keys are converted to a more friendly representation for example,\ninstead of sending \"\u001BA\" which is provided by the terminal, we convert it to \"ArrowUp\".\n\nAny other messages sent to the process will be handled using `handle_info/2`:\n\n```\ndef handle_info(:some_message, term), do: {:noreply, term}\n```","ref":"Breeze.View.html#module-handling-events"},{"type":"module","title":"Style - Breeze.View","doc":"The `style` attribute can be used to style the box. This uses `BackBreeze.Box.new/1` under\nthe hood.\n\nA box can be styled similar to CSS using the style attribute:\n\n```\n Hello World \n```\n\nThe following styles are supported:\n\n * `border` - add a line border to the box\n * `bold` - make the text bold\n * `italic` - make the text italic\n * `inverse` - reverse the foreground-background\n * `reverse` - reverse the foreground-background\n * `inline` - display the elements inline (join horizontally)\n * `width-x` - set the width of the element\n * `height-x` - set the height of the element\n * `absolute` - position the elements absolute relative to the parent\n * `border-x` - set the border color where x is a number\n * `bg-x` - set the background color where x is a number\n * `text-x` - set the foreground color where x is a number","ref":"Breeze.View.html#module-style"},{"type":"module","title":"Implicits - Breeze.View","doc":"Implicits provide a way of adding event handler/state that exists outside of the view\nthese should be abstracted out into their own components.\n\nFor example, consider a list component:\n\n```\ndef render(assigns) do\n~H\"\"\"\n<.list id={id} br-change=\"my_custom_event\">\n<:item value=\"hello\">Hello</:item>\n<:item value=\"world\">World</:item>\n<:item value=\"foo\">Foo</:item>\n</.list>\n\"\"\"\n\ndef handle_event(\"my_custom_event\", %{value: value}, term), do: ...\n```\n\nIdeally, we don't want to have to keep track of the selected value, handle key events,\netc within our view. In our view, we might only care about the selected value. In this case, we can define the list component to use an implicit state module.\n\n```\nattr :id, :string, required: true\nattr :rest, :global\n\nslot :item do\n  attr(:value, :string, required: true)\nend\n\ndef list(assigns) do\n  ~H\"\"\"\n   \n     <%= render_slot(item, %{}) %> \n   \n  \"\"\"\nend\n  ```\n\nThe implicit module is first called with an `init/2` callback, which is called with all\nthe attributes of all the child elements, and the previous state. This ensures that\nvalues persist between renders.\n\n```\ndefmodule MyAppList do\n\n  def init(children, last_state) do\n    %{values: Enum.map(children, &(&1.value)), selected: last_state[:selected]}\n  end\nend\n```\n\nThere is also a `handle_event/3` callback. This is similar to the callback for a view, but\nreturns different values. Here we handle key events and return a `:change` event along\nwith the new state. The `:change` will be used by `br-change` to pass through to the handle_event\ncallback of the Breeze.View.\n\n```\ndef handle_event(_, %{\"key\" => \"ArrowDown\"}, %{values: values} = state) do\n  index = Enum.find_index(values, &(&1 == state.selected))\n  value = if index, do: Enum.at(values, index + 1) || hd(values), else: hd(values)\n  {{:change, %{value: value}}, %{state | selected: value}}\nend\n\ndef handle_event(_, %{\"key\" => \"ArrowUp\"}, %{values: values} = state) do\n  index = Enum.find_index(values, &(&1 == state.selected))\n  first = hd(Enum.reverse(values))\n  value = if index, do: Enum.at(values, index - 1) || first, else: first\n  {{:change, %{value: value}}, %{state | selected: value}}\nend\n\ndef handle_event(_, _, state), do: {:noreply, state}\n```\n\nThere is one final handler which is called when a child is being rendered.\n`handle_modifiers/2` can be used to tell the renderer things about the state.\nIn this case, we want to add the `:selected` flag so that the list element is\nstyled differently.\n\n```\ndef handle_modifiers(attributes, state) do\n  if state.selected == Keyword.get(attributes, :value) do\n    [selected: true]\n  else\n    []\n  end\nend\n```","ref":"Breeze.View.html#module-implicits"},{"type":"function","title":"Breeze.View.assign/2","doc":"","ref":"Breeze.View.html#assign/2"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}